@@ -32,7 +32,7 @@
 
 from reportlab.platypus.flowables import Flowable
 from reportlab.lib.units import inch
-import string
+from string import ascii_lowercase, ascii_uppercase, digits as string_digits
 
 class Barcode(Flowable):
     """Abstract Base for barcodes. Includes implementations of
@@ -46,7 +46,7 @@ class Barcode(Flowable):
         return self.encoded
 
     def __init__(self, value='',**kwd):
-        self.value = value
+        self.value = str(value)
 
         self._setKeywords(**kwd)
         if not hasattr(self, 'gap'):
@@ -60,7 +60,7 @@ class Barcode(Flowable):
         self.computeSize()
 
     def _setKeywords(self,**kwd):
-        for (k, v) in kwd.iteritems():
+        for (k, v) in kwd.items():
             setattr(self, k, v)
 
     def validate(self):
@@ -103,15 +103,21 @@ class Barcode(Flowable):
         self._height = self.barHeight
         self._width = w
 
+    @property
     def width(self):
         self._calculate()
         return self._width
-    width = property(width)
+    @width.setter
+    def width(self,v):
+        pass
 
+    @property
     def height(self):
         self._calculate()
         return self._height
-    height = property(height)
+    @height.setter
+    def height(self,v):
+        pass
 
     def draw(self):
         self._calculate()
@@ -138,10 +144,21 @@ class Barcode(Flowable):
                 left = left + wx
 
         if self.bearers:
-            self.rect(self.lquiet, 0, \
-                self._width - (self.lquiet + self.rquiet), b)
-            self.rect(self.lquiet, self.barHeight - b, \
-                self._width - (self.lquiet + self.rquiet), b)
+            if getattr(self,'bearerBox', None):
+                canv = self.canv
+                if hasattr(canv,'_Gadd'):
+                    #this is a widget rect takes other arguments
+                    canv.rect(bb, bb, self.width, self.barHeight-b,
+                            strokeWidth=b, strokeColor=self.barFillColor or self.barStrokeColor, fillColor=None)
+                else:
+                    canv.saveState()
+                    canv.setLineWidth(b)
+                    canv.rect(bb, bb, self.width, self.barHeight-b, stroke=1, fill=0)
+                    canv.restoreState()
+            else:
+                w = self._width - (self.lquiet + self.rquiet)
+                self.rect(self.lquiet, 0, w, b)
+                self.rect(self.lquiet, self.barHeight - b, w, b)
 
         self.drawHumanReadable()
 
@@ -176,6 +193,12 @@ class Barcode(Flowable):
         getattr(canv,func)(x,y,text)
         canv.restoreState()
 
+    def _checkVal(self, name, v, allowed):
+        if v not in allowed:
+            raise ValueError('%s attribute %s is invalid %r\nnot in allowed %r' % (
+                self.__class__.__name__, name, v, allowed))
+        return v
+
 class MultiWidthBarcode(Barcode):
     """Base for variable-bar-width codes like Code93 and Code128"""
 
@@ -187,9 +210,9 @@ class MultiWidthBarcode(Barcode):
 
         for c in self.decomposed:
             oc = ord(c)
-            if c in string.lowercase:
+            if c in ascii_lowercase:
                 w = w + barWidth * (oc - oa)
-            elif c in string.uppercase:
+            elif c in ascii_uppercase:
                 w = w + barWidth * (oc - oA)
 
         if self.barHeight is None:
@@ -210,9 +233,9 @@ class MultiWidthBarcode(Barcode):
 
         for c in self.decomposed:
             oc = ord(c)
-            if c in string.lowercase:
+            if c in ascii_lowercase:
                 left = left + (oc - oa) * barWidth
-            elif c in string.uppercase:
+            elif c in ascii_uppercase:
                 w = (oc - oA) * barWidth
                 self.rect(left, 0, w, self.barHeight)
                 left += w
@@ -225,7 +248,7 @@ class I2of5(Barcode):
 
     Options that may be passed to constructor:
 
-        value (int, or numeric string. required.):
+        value (int, or numeric string required.):
             The value to encode.
 
         barWidth (float, default .0075):
@@ -254,6 +277,9 @@ class I2of5(Barcode):
             Set to zero for no bearer bars. (Bearer bars help detect
             misscans, so it is suggested to leave them on).
 
+        bearerBox (bool default False)
+            if true draw a  true rectangle of width bearers around the barcode.
+
         quiet (bool, default 1):
             Whether to include quiet zones in the symbol.
 
@@ -298,6 +324,7 @@ class I2of5(Barcode):
     ratio = 2.2
     checksum = 1
     bearers = 3.0
+    bearerBox = False
     quiet = 1
     lquiet = None
     rquiet = None
@@ -308,7 +335,7 @@ class I2of5(Barcode):
         if type(value) == type(1):
             value = str(value)
 
-        for (k, v) in args.items():
+        for k, v in args.items():
             setattr(self, k, v)
 
         if self.quiet:
@@ -323,8 +350,8 @@ class I2of5(Barcode):
     def validate(self):
         vval = ""
         self.valid = 1
-        for c in string.strip(self.value):
-            if c not in string.digits:
+        for c in self.value.strip():
+            if c not in string_digits:
                 self.valid = 0
                 continue
             vval = vval + c
@@ -342,7 +369,7 @@ class I2of5(Barcode):
             c += 1
 
         if cs:
-            c = 3*sum([int(s[i]) for i in xrange(0,c,2)])+sum([int(s[i]) for i in xrange(1,c,2)])
+            c = 3*sum([int(s[i]) for i in range(0,c,2)])+sum([int(s[i]) for i in range(1,c,2)])
             s += str((10 - c) % 10)
 
         self.encoded = s
@@ -351,7 +378,7 @@ class I2of5(Barcode):
         dval = self.stop and [self.patterns['start']] or []
         a = dval.append
 
-        for i in xrange(0, len(self.encoded), 2):
+        for i in range(0, len(self.encoded), 2):
             b = self.patterns['B' + self.encoded[i]]
             s = self.patterns['S' + self.encoded[i+1]]
 
@@ -368,7 +395,7 @@ class MSI(Barcode):
 
     Options that may be passed to constructor:
 
-        value (int, or numeric string. required.):
+        value (int, or numeric string required.):
             The value to encode.
 
         barWidth (float, default .0075):
@@ -433,7 +460,7 @@ class MSI(Barcode):
         if type(value) == type(1):
             value = str(value)
 
-        for (k, v) in args.items():
+        for k, v in args.items():
             setattr(self, k, v)
 
         if self.quiet:
@@ -448,8 +475,8 @@ class MSI(Barcode):
     def validate(self):
         vval = ""
         self.valid = 1
-        for c in string.strip(self.value):
-            if c not in string.digits:
+        for c in self.value.strip():
+            if c not in string_digits:
                 self.valid = 0
                 continue
             vval = vval + c
@@ -489,7 +516,7 @@ class Codabar(Barcode):
 
     Options that may be passed to constructor:
 
-        value (string. required.):
+        value (string required.):
             The value to encode.
 
         barWidth (float, default .0065):
@@ -553,7 +580,7 @@ class Codabar(Barcode):
         '+' : 15,   'A' : 16,   'B' : 17,   'C' : 18,   'D' : 19
         }
 
-    chars = string.digits + "-$:/.+"
+    chars = string_digits + "-$:/.+"
 
     stop = 1
     barHeight = None
@@ -569,7 +596,7 @@ class Codabar(Barcode):
         if type(value) == type(1):
             value = str(value)
 
-        for (k, v) in args.items():
+        for k, v in args.items():
             setattr(self, k, v)
 
         if self.quiet:
@@ -584,7 +611,7 @@ class Codabar(Barcode):
     def validate(self):
         vval = ""
         self.valid = 1
-        s = string.strip(self.value)
+        s = self.value.strip()
         for i in range(0, len(s)):
             c = s[i]
             if c not in self.chars:
@@ -621,7 +648,7 @@ class Code11(Barcode):
     Code 11 is an almost-numeric barcode. It encodes the digits 0-9 plus
     dash ("-"). 11 characters total, hence the name.
 
-        value (int or string. required.):
+        value (int or string required.):
             The value to encode.
 
         barWidth (float, default .0075):
@@ -689,7 +716,7 @@ class Code11(Barcode):
         if type(value) == type(1):
             value = str(value)
 
-        for (k, v) in args.items():
+        for k, v in args.items():
             setattr(self, k, v)
 
         if self.quiet:
@@ -704,8 +731,8 @@ class Code11(Barcode):
     def validate(self):
         vval = ""
         self.valid = 1
-        s = string.strip(self.value)
-        for i in xrange(0, len(s)):
+        s = self.value.strip()
+        for i in range(0, len(s)):
             c = s[i]
             if c not in self.chars:
                 self.Valid = 0
